// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.

package pm

import (
	"LAPP_ACURA_MOM_BACKEND/global"
	"LAPP_ACURA_MOM_BACKEND/grmi"
	model "LAPP_ACURA_MOM_BACKEND/models/pm"
	"LAPP_ACURA_MOM_BACKEND/utils"
	"LAPP_ACURA_MOM_BACKEND/web/middleware/jwts"
	"LAPP_ACURA_MOM_BACKEND/web/supports"
	"encoding/json"
	"github.com/kataras/iris/v12"
	"github.com/kataras/iris/v12/core/router"
	"log"
)

/******************************************************************************
 *
 * @Function Name            : RegisterOneAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理单条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterOneAssetType(party router.Party, httpMethod string, path string, method func(*global.User, *model.AssetType) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		//获取通过iris.WithPostMaxMemory获取的最大上传值大小。
		maxSize := ctx.Application().ConfigurationReadOnly().GetPostMaxMemory()
		err := ctx.Request().ParseMultipartForm(maxSize)
		if err != nil {
			ctx.StatusCode(iris.StatusInternalServerError)
			ctx.WriteString(err.Error())
			return
		}
		form := ctx.Request().MultipartForm
		files := form.File["upload[]"]

		entity := new(model.AssetType)
		info := ctx.FormValue("data")
		err = json.Unmarshal([]byte(info), entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, "json解析错误:"+err.Error(), nil)
			return
		}

		if entity == nil {
			supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到原有信息:"+err.Error(), nil)
			return
		}

		for _, file := range files {
			if !utils.ValueIsEmpty(file.Filename) {
				guid := utils.MakeOrderSn(user.UserId)
				filPath := "public/uploadfile/" + guid + ".jpg"
				savePath := "web/public/uploadfile/" + guid + ".jpg"
				// 上传文件至指定目录
				err := utils.SaveUploadedFile(file, savePath)
				if err != nil {
					log.Printf("上传图片错误:%v", err)
					supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
					return
				}
				entity.Bitmap = filPath
			}
		}

		err = method(user, entity)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterIDOfAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个通过主键处理单条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterIDOfAssetType(party router.Party, httpMethod string, path string, method func(*global.User, string) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		var err error = nil
		assetType := ctx.Params().GetString("assetType")

		err = method(user, assetType)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertOneAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入单条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterInsertOneAssetType(party router.Party, path string, method func(*global.User, *model.AssetType) error) {

	RegisterOneAssetType(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteOneAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除单条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterDeleteOneAssetType(party router.Party, path string, method func(*global.User, string) error) {

	RegisterIDOfAssetType(party, "DELETE", path+"/{assetType:string}", method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectOneAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查找单条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterSelectOneAssetType(party router.Party, path string, method func(*global.User, string) (*model.AssetType, error)) {

	party.Get(path+"/{assetType:string}", func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		assetType := ctx.Params().GetString("assetType")
		result, err := method(user, assetType)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		if result == nil {
			supports.Error(ctx, iris.StatusNotFound, supports.NotFound, nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateOneAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改单条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterUpdateOneAssetType(party router.Party, path string, method func(*global.User, *model.AssetType) error) {

	RegisterOneAssetType(party, "PUT", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterMultiAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个处理多条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : HTTP方法
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterMultiAssetType(party router.Party, httpMethod string, path string, method func(*global.User, *[]model.AssetType) error) {

	party.Handle(httpMethod, path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		entities := make([]model.AssetType, 0, 10)
		err := ctx.ReadJSON(&entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}

		err = method(user, &entities)
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, nil)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterInsertAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个插入多条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterInsertAssetType(party router.Party, path string, method func(*global.User, *[]model.AssetType) error) {

	RegisterMultiAssetType(party, "POST", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterDeleteAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个删除多条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterDeleteAssetType(party router.Party, path string, method func(*global.User, *[]model.AssetType) error) {

	RegisterMultiAssetType(party, "DELETE", path, method)
}

/******************************************************************************
 *
 * @Function Name            : RegisterSelectAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterSelectAssetType(party router.Party, path string, method func(*global.User, map[string]string) ([]model.AssetType, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterQueryAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个查询AssetType并分页的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterQueryAssetType(party router.Party, path string, method func(*global.User, map[string]string) (grmi.PagingResult, error)) {

	party.Get(path, func(ctx iris.Context) {
		user, ok := jwts.ParseToken(ctx)
		if !ok {
			return
		}

		result, err := method(user, ctx.URLParams())
		if err != nil {
			supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
			return
		}
		supports.Ok(ctx, supports.OptionSuccess, result)
	})
}

/******************************************************************************
 *
 * @Function Name            : RegisterUpdateAssetType
 *-----------------------------------------------------------------------------
 *
 * @Description              : 为一个修改多条AssetType的方法注册路由
 *
 * @Function Parameters      : 路由分组
 *
 * @Function Parameters      : 路径
 *
 * @Function Parameters      : 实际处理请求的方法
 *
 * @Author                   : 代码生成器创建
 *
 * @Date                     : 2021-10-08 13:20:42
 *
 ******************************************************************************/
func RegisterUpdateAssetType(party router.Party, path string, method func(*global.User, *[]model.AssetType) error) {

	RegisterMultiAssetType(party, "PUT", path, method)
}
